//+------------------------------------------------------------------+
//|                                                                  |
//| "Fixing the Bollinger bands"                                     |
//| Futures magazine 05.2010, David Rooke                            }
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1  DeepSkyBlue 
#property indicator_color2  PaleVioletRed
#property indicator_width1  1
#property indicator_width2  1

//
//
//
//
//

extern int    Length                 = 120;//20 
extern int    Price                  = PRICE_CLOSE;
extern double Deviations             = 2.0;
extern bool   UseClassicalDeviations = false; 
extern double SmoothLength           = 1.0;
extern double SmoothPhase            = 0;
extern double LevelUp                = 1.0;
extern double LevelDown              = 0.0;
extern bool   ShowAllLevelBreaks     = true;
extern bool   ShowReversals          = false;

//
//
//
//
//

double vbp[];
double vbc[];
double vbu[];
double vbd[];
double prices[];
double arrowUp[];
double arrowDn[];
double state[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
      SetIndexBuffer(0, arrowUp); SetIndexStyle(0,DRAW_ARROW); SetIndexArrow(0,241);
      SetIndexBuffer(1, arrowDn); SetIndexStyle(1,DRAW_ARROW); SetIndexArrow(1,242);
      SetIndexBuffer(2, vbp); 
      SetIndexBuffer(3, vbc); 
      SetIndexBuffer(4, vbu); 
      SetIndexBuffer(5, vbd); 
      SetIndexBuffer(6, prices); 
      SetIndexBuffer(7, state); 
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //

   for(i=limit; i>=0; i--)
   {
      prices[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);

         double tema1 = iTema(prices[i],Length,i,0);
         double tema2 = iTema(tema1    ,Length,i,3);

      vbc[i] = 2.0*tema1-tema2;

         double deviation = iDeviationPlus(prices,vbc,Length,i);

      vbu[i] = vbc[i]+deviation*Deviations;
      vbd[i] = vbc[i]-deviation*Deviations;

         double width = vbu[i]-vbd[i];

      if (width>0)
           vbp[i] = iSmooth((prices[i]-vbd[i])/width,SmoothLength,SmoothPhase,i);
      else vbp[i] = iSmooth(0                       ,SmoothLength,SmoothPhase,i);
      
      //
      //
      //
      //
      //
      
      arrowUp[i] = EMPTY_VALUE;
      arrowDn[i] = EMPTY_VALUE;
      state[i]   = state[i+1];
         if (vbp[i] > LevelUp)                                             state[i] =  1;
         if (vbp[i] < LevelDown)                                           state[i] = -1;
         if (ShowAllLevelBreaks && vbp[i] > LevelDown && vbp[i] < LevelUp) state[i] =  0;
         if (state[i] != state[i+1])
         {
            if (state[i]== 1) arrowUp[i] = Low[i]  - iATR(NULL,0,20,i)/2.0;
            if (state[i]==-1) arrowDn[i] = High[i] + iATR(NULL,0,20,i)/2.0;
            if (ShowAllLevelBreaks && ShowReversals)
            {
               if (state[i+1]== 1 && state[i]!= 1) arrowDn[i] = High[i] + iATR(NULL,0,20,i)/2.0;
               if (state[i+1]==-1 && state[i]!=-1) arrowUp[i] = Low[i]  - iATR(NULL,0,20,i)/2.0;
            }
         }
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrk[][6];

double iTema(double price, int length, int i, int s=0)
{
   if (ArrayRange(wrk,0)!=Bars) ArrayResize(wrk,Bars);
   
   //
   //
   //
   //
   //
   
   double alpha = 2.0 /(1.0 + length);
   int    r = Bars-i-1;
   
   if (r < 1)
      {
         wrk[r][0+s] = price;
         wrk[r][1+s] = price;
         wrk[r][2+s] = price;
      }
   else
      {
         wrk[r][0+s] = wrk[r-1][0+s]+alpha*(price      -wrk[r-1][0+s]);
         wrk[r][1+s] = wrk[r-1][1+s]+alpha*(wrk[r][0+s]-wrk[r-1][1+s]);
         wrk[r][2+s] = wrk[r-1][2+s]+alpha*(wrk[r][1+s]-wrk[r-1][2+s]);
      }
   return(3*wrk[r][0+s] - 3*wrk[r][1+s] + wrk[r][2+s]);
}

//
//
//
//    In the original article the second method is how deviation is
//    calculated . It is a questionableble method , but if we apply
//    standard calculation the bands tend to be "spiky".That is the
//    reason to have the "UseClassicalDeviations" option
//
//
//

double iDeviationPlus(double& array[], double& ma[], int period, int i)
{
   double sum = 0;
      if (UseClassicalDeviations)
            for(int k=0; k<period; k++) sum += (array[i+k]-ma[i])  *(array[i+k]-ma[i]);
      else  for(    k=0; k<period; k++) sum += (array[i+k]-ma[i+k])*(array[i+k]-ma[i+k]);       
   return(MathSqrt(sum/period));
}

//+------------------------------------------------------------------
//|                                                                 
//+------------------------------------------------------------------
//
//
//
//
//

double wrkj[][10];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

//
//
//
//
//

double iSmooth(double price, double length, double phase, int i, int s=0)
{
   if (ArrayRange(wrkj,0) != Bars) ArrayResize(wrkj,Bars);
   if (length <=1) return(price);
   
   int r = Bars-i-1; 
      if (r==0) { for(int k=0; k<7; k++) wrkj[0][k+s]=price; for(; k<10; k++) wrkj[0][k+s]=0; return(price); }

   //
   //
   //
   //
   //
   
      double len1   = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1   = MathMax(len1-2.0,0.5);
      double del1   = price - wrkj[r-1][bsmax+s];
      double del2   = price - wrkj[r-1][bsmin+s];
      double div    = 1.0/(10.0+10.0*(MathMin(MathMax(length-10,0),100))/100);
      int    forBar = MathMin(r,10);
	
         wrkj[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrkj[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrkj[r][volty+s] = MathAbs(del2); 
         wrkj[r][vsum+s] =	wrkj[r-1][vsum+s] + (wrkj[r][volty+s]-wrkj[r-forBar][volty+s])*div;
         
         //
         //
         //
         //
         //
   
         wrkj[r][avolty+s] = wrkj[r-1][avolty+s]+(2.0/(MathMax(4.0*length,30)+1.0))*(wrkj[r][vsum+s]-wrkj[r-1][avolty+s]);
            if (wrkj[r][avolty+s] > 0)
               double dVolty = wrkj[r][volty+s]/wrkj[r][avolty+s]; else dVolty = 0;   
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));

         if (del1 > 0) wrkj[r][bsmax+s] = price; else wrkj[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrkj[r][bsmin+s] = price; else wrkj[r][bsmin+s] = price - Kv*del2;
	
   //
   //
   //
   //
   //
      
      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrkj[r][0+s] = price + alpha*(wrkj[r-1][0+s]-price);
         wrkj[r][1+s] = (price - wrkj[r][0+s])*(1-beta) + beta*wrkj[r-1][1+s];
         wrkj[r][2+s] = (wrkj[r][0+s] + R*wrkj[r][1+s]);
         wrkj[r][3+s] = (wrkj[r][2+s] - wrkj[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrkj[r-1][3+s];
         wrkj[r][4+s] = (wrkj[r-1][4+s] + wrkj[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrkj[r][4+s]);
}